@@ -155,6 +155,7 @@ class MQTTTransport(object):
             )
 
         if self._proxy_options:
+            logger.info("Setting custom proxy options on mqtt client")
             mqtt_client.proxy_set(
                 proxy_type=self._proxy_options.proxy_type,
                 proxy_addr=self._proxy_options.proxy_address,
@@ -210,16 +211,25 @@ class MQTTTransport(object):
             if rc:  # i.e. if there is an error
                 logger.debug("".join(traceback.format_stack()))
                 cause = _create_error_from_rc_code(rc)
-                this._cleanup_transport_on_error()
+                if this:
+                    this._cleanup_transport_on_error()
 
-            if this.on_mqtt_disconnected_handler:
-                try:
-                    this.on_mqtt_disconnected_handler(cause)
-                except Exception:
-                    logger.error("Unexpected error calling on_mqtt_disconnected_handler")
-                    logger.error(traceback.format_exc())
+            if not this:
+                # Paho will sometimes call this after we've been garbage collected,  If so, we have to
+                # stop the loop to make sure the Paho thread shuts down.
+                logger.info(
+                    "on_disconnect called with transport==None. Transport must have been garbage collected. stopping loop"
+                )
+                client.loop_stop()
             else:
-                logger.warning("No event handler callback set for on_mqtt_disconnected_handler")
+                if this.on_mqtt_disconnected_handler:
+                    try:
+                        this.on_mqtt_disconnected_handler(cause)
+                    except Exception:
+                        logger.error("Unexpected error calling on_mqtt_disconnected_handler")
+                        logger.error(traceback.format_exc())
+                else:
+                    logger.warning("No event handler callback set for on_mqtt_disconnected_handler")
 
         def on_subscribe(client, userdata, mid, granted_qos):
             this = self_weakref()
@@ -318,12 +328,15 @@ class MQTTTransport(object):
         ssl_context = ssl.SSLContext(protocol=ssl.PROTOCOL_TLSv1_2)
 
         if self._server_verification_cert:
+            logger.debug("configuring SSL context with custom server verification cert")
             ssl_context.load_verify_locations(cadata=self._server_verification_cert)
         else:
+            logger.debug("configuring SSL context with default certs")
             ssl_context.load_default_certs()
 
         if self._cipher:
             try:
+                logger.debug("configuring SSL context with cipher suites")
                 ssl_context.set_ciphers(self._cipher)
             except ssl.SSLError as e:
                 # TODO: custom error with more detail?
@@ -417,37 +430,6 @@ class MQTTTransport(object):
             raise _create_error_from_rc_code(rc)
         self._mqtt_client.loop_start()
 
-    def reauthorize_connection(self, password=None):
-        """
-        Reauthorize with the MQTT broker, using username set at instantiation.
-
-        Connect should have previously been called in order to use this function.
-
-        The password is not required if the transport was instantiated with an x509 certificate.
-
-        :param str password: The password for reauthorizing with the MQTT broker (Optional).
-
-        :raises: ConnectionFailedError if connection could not be established.
-        :raises: ConnectionDroppedError if connection is dropped during execution.
-        :raises: UnauthorizedError if there is an error authenticating.
-        :raises: ProtocolClientError if there is some other client error.
-        """
-        logger.info("reauthorizing MQTT client")
-        self._mqtt_client.username_pw_set(username=self._username, password=password)
-        try:
-            rc = self._mqtt_client.reconnect()
-        except Exception as e:
-            logger.info("reconnect raised {}".format(e))
-            self._cleanup_transport_on_error()
-            raise exceptions.ConnectionDroppedError(
-                message="Unexpected Paho failure during reconnect", cause=e
-            )
-        logger.debug("_mqtt_client.reconnect returned rc={}".format(rc))
-        if rc:
-            # This could result in ConnectionFailedError, ConnectionDroppedError, UnauthorizedError
-            # or ProtocolClientError
-            raise _create_error_from_rc_code(rc)
-
     def disconnect(self):
         """
         Disconnect from the MQTT broker.
