@@ -24,7 +24,7 @@ from azure.iot.device.common.callable_weak_method import CallableWeakMethod
 
 logger = logging.getLogger(__name__)
 
-# Maximum amount of time we wait for ConnectOperation or ReauthorizeConnectionOperation to complete
+# Maximum amount of time we wait for ConnectOperation to complete
 WATCHDOG_INTERVAL = 10
 
 
@@ -38,9 +38,6 @@ class MQTTTransportStage(PipelineStage):
     def __init__(self):
         super(MQTTTransportStage, self).__init__()
 
-        # The sas_token will be set when Connetion Args are received
-        self.sas_token = None
-
         # The transport will be instantiated when Connection Args are received
         self.transport = None
 
@@ -111,17 +108,33 @@ class MQTTTransportStage(PipelineStage):
 
     @pipeline_thread.runs_on_pipeline_thread
     def _run_op(self, op):
-        if isinstance(op, pipeline_ops_mqtt.SetMQTTConnectionArgsOperation):
-            # pipeline_ops_mqtt.SetMQTTConnectionArgsOperation is where we create our MQTTTransport object and set
-            # all of its properties.
+        if isinstance(op, pipeline_ops_base.InitializePipelineOperation):
+
+            # If there is a gateway hostname, use that as the hostname for connection,
+            # rather than the hostname itself
+            if self.pipeline_root.pipeline_configuration.gateway_hostname:
+                logger.debug(
+                    "Gateway Hostname Present. Setting Hostname to: {}".format(
+                        self.pipeline_root.pipeline_configuration.gateway_hostname
+                    )
+                )
+                hostname = self.pipeline_root.pipeline_configuration.gateway_hostname
+            else:
+                logger.debug(
+                    "Gateway Hostname not present. Setting Hostname to: {}".format(
+                        self.pipeline_root.pipeline_configuration.hostname
+                    )
+                )
+                hostname = self.pipeline_root.pipeline_configuration.hostname
+
+            # Create the Transport object, set it's handlers
             logger.debug("{}({}): got connection args".format(self.name, op.name))
-            self.sas_token = op.sas_token
             self.transport = MQTTTransport(
                 client_id=op.client_id,
-                hostname=op.hostname,
+                hostname=hostname,
                 username=op.username,
-                server_verification_cert=op.server_verification_cert,
-                x509_cert=op.client_cert,
+                server_verification_cert=self.pipeline_root.pipeline_configuration.server_verification_cert,
+                x509_cert=self.pipeline_root.pipeline_configuration.x509,
                 websockets=self.pipeline_root.pipeline_configuration.websockets,
                 cipher=self.pipeline_root.pipeline_configuration.cipher,
                 proxy_options=self.pipeline_root.pipeline_configuration.proxy_options,
@@ -153,19 +166,20 @@ class MQTTTransportStage(PipelineStage):
 
             op.complete()
 
-        elif isinstance(op, pipeline_ops_base.UpdateSasTokenOperation):
-            logger.debug("{}({}): saving sas token and completing".format(self.name, op.name))
-            self.sas_token = op.sas_token
-            op.complete()
-
         elif isinstance(op, pipeline_ops_base.ConnectOperation):
             logger.info("{}({}): connecting".format(self.name, op.name))
 
             self._cancel_pending_connection_op()
             self._pending_connection_op = op
             self._start_connection_watchdog(op)
+            # Use SasToken as password if present. If not present (e.g. using X509),
+            # then no password is required because auth is handled via other means.
+            if self.pipeline_root.pipeline_configuration.sastoken:
+                password = str(self.pipeline_root.pipeline_configuration.sastoken)
+            else:
+                password = None
             try:
-                self.transport.connect(password=self.sas_token)
+                self.transport.connect(password=password)
             except Exception as e:
                 logger.error("transport.connect raised error")
                 logger.error(traceback.format_exc())
@@ -173,32 +187,10 @@ class MQTTTransportStage(PipelineStage):
                 self._pending_connection_op = None
                 op.complete(error=e)
 
-        elif isinstance(op, pipeline_ops_base.ReauthorizeConnectionOperation):
-            logger.info("{}({}): reauthorizing".format(self.name, op.name))
-
-            # We set _active_connect_op here because reauthorizing the connection is the same as a connect for "active operation" tracking purposes.
-            self._cancel_pending_connection_op()
-            self._pending_connection_op = op
-            self._start_connection_watchdog(op)
-            try:
-                self.transport.reauthorize_connection(password=self.sas_token)
-            except Exception as e:
-                logger.error("transport.reauthorize_connection raised error")
-                logger.error(traceback.format_exc())
-                self._cancel_connection_watchdog(op)
-                self._pending_connection_op = None
-                # Send up a DisconenctedEvent.  If we ran a ReauthorizeConnectionOperatoin,
-                # some code must think we're still connected.  If we got an exception here,
-                # we're not conencted, and we need to notify upper layers. (Paho should do this,
-                # but it only causes a DisconnectedEvent on manual disconnect or if a PINGRESP
-                # failed, and it's possible to hit this code without either of those things
-                # happening.
-                if isinstance(e, transport_exceptions.ConnectionDroppedError):
-                    self.send_event_up(pipeline_events_base.DisconnectedEvent())
-                op.complete(error=e)
-
-        elif isinstance(op, pipeline_ops_base.DisconnectOperation):
-            logger.info("{}({}): disconnecting".format(self.name, op.name))
+        elif isinstance(op, pipeline_ops_base.DisconnectOperation) or isinstance(
+            op, pipeline_ops_base.ReauthorizeConnectionOperation
+        ):
+            logger.info("{}({}): disconnecting or reauthorizing".format(self.name, op.name))
 
             self._cancel_pending_connection_op()
             self._pending_connection_op = op
@@ -269,6 +261,7 @@ class MQTTTransportStage(PipelineStage):
         Handler that gets called by the protocol library when an incoming message arrives.
         Convert that message into a pipeline event and pass it up for someone to handle.
         """
+        logger.debug("{}: message received on topic {}".format(self.name, topic))
         self.send_event_up(
             pipeline_events_mqtt.IncomingMQTTMessageEvent(topic=topic, payload=payload)
         )
@@ -283,11 +276,7 @@ class MQTTTransportStage(PipelineStage):
         # we do anything else (in case upper stages have any "are we connected" logic.
         self.send_event_up(pipeline_events_base.ConnectedEvent())
 
-        if isinstance(
-            self._pending_connection_op, pipeline_ops_base.ConnectOperation
-        ) or isinstance(
-            self._pending_connection_op, pipeline_ops_base.ReauthorizeConnectionOperation
-        ):
+        if isinstance(self._pending_connection_op, pipeline_ops_base.ConnectOperation):
             logger.debug("completing connect op")
             op = self._pending_connection_op
             self._cancel_connection_watchdog(op)
@@ -309,11 +298,7 @@ class MQTTTransportStage(PipelineStage):
 
         logger.info("{}: _on_mqtt_connection_failure called: {}".format(self.name, cause))
 
-        if isinstance(
-            self._pending_connection_op, pipeline_ops_base.ConnectOperation
-        ) or isinstance(
-            self._pending_connection_op, pipeline_ops_base.ReauthorizeConnectionOperation
-        ):
+        if isinstance(self._pending_connection_op, pipeline_ops_base.ConnectOperation):
             logger.debug("{}: failing connect op".format(self.name))
             op = self._pending_connection_op
             self._cancel_connection_watchdog(op)
@@ -352,7 +337,9 @@ class MQTTTransportStage(PipelineStage):
             self._cancel_connection_watchdog(op)
             self._pending_connection_op = None
 
-            if isinstance(op, pipeline_ops_base.DisconnectOperation):
+            if isinstance(op, pipeline_ops_base.DisconnectOperation) or isinstance(
+                op, pipeline_ops_base.ReauthorizeConnectionOperation
+            ):
                 # Swallow any errors if we intended to disconnect - even if something went wrong, we
                 # got to the state we wanted to be in!
                 if cause:
