@@ -11,23 +11,17 @@ import abc
 import logging
 import os
 import io
-from . import auth
 from . import pipeline
+from azure.iot.device.common.auth import connection_string as cs
+from azure.iot.device.common.auth import sastoken as st
+
+from azure.iot.device.common import auth
+from . import edge_hsm
 
 logger = logging.getLogger(__name__)
 
-# A note on implementation:
-# The intializer methods accept pipeline(s) instead of an auth provider in order to protect
-# the client from logic related to authentication providers. This reduces edge cases, and allows
-# pipeline configuration to be specifically tailored to the method of instantiation.
-# For instance, .create_from_connection_string and .create_from_edge_envrionment both can use
-# SymmetricKeyAuthenticationProviders to instantiate pipeline(s), but only .create_from_edge_environment
-# should use it to instantiate an HTTPPipeline. If the initializer accepted an auth provider, and then
-# used it to create pipelines, this detail would be lost, as there would be no way to tell if a
-# SymmetricKeyAuthenticationProvider was intended to be part of an Edge scenario or not.
 
-
-def _validate_kwargs(**kwargs):
+def _validate_kwargs(exclude=[], **kwargs):
     """Helper function to validate user provided kwargs.
     Raises TypeError if an invalid option has been provided"""
     valid_kwargs = [
@@ -36,25 +30,38 @@ def _validate_kwargs(**kwargs):
         "cipher",
         "server_verification_cert",
         "proxy_options",
+        "sastoken_ttl",
     ]
 
     for kwarg in kwargs:
-        if kwarg not in valid_kwargs:
-            raise TypeError("Got an unexpected keyword argument '{}'".format(kwarg))
+        if (kwarg not in valid_kwargs) or (kwarg in exclude):
+            raise TypeError("Unsupported keyword argument: '{}'".format(kwarg))
 
 
-def _get_pipeline_config_kwargs(**kwargs):
-    """Helper function to get a subset of user provided kwargs relevant to IoTHubPipelineConfig"""
-    new_kwargs = {}
-    if "product_info" in kwargs:
-        new_kwargs["product_info"] = kwargs["product_info"]
-    if "websockets" in kwargs:
-        new_kwargs["websockets"] = kwargs["websockets"]
-    if "cipher" in kwargs:
-        new_kwargs["cipher"] = kwargs["cipher"]
-    if "proxy_options" in kwargs:
-        new_kwargs["proxy_options"] = kwargs["proxy_options"]
-    return new_kwargs
+def _get_config_kwargs(**kwargs):
+    """Get the subset of kwargs which pertain the config object"""
+    valid_config_kwargs = [
+        "product_info",
+        "websockets",
+        "cipher",
+        "server_verification_cert",
+        "proxy_options",
+    ]
+
+    config_kwargs = {}
+    for kwarg in kwargs:
+        if kwarg in valid_config_kwargs:
+            config_kwargs[kwarg] = kwargs[kwarg]
+    return config_kwargs
+
+
+def _form_sas_uri(hostname, device_id, module_id=None):
+    if module_id:
+        return "{hostname}/devices/{device_id}/modules/{module_id}".format(
+            hostname=hostname, device_id=device_id, module_id=module_id
+        )
+    else:
+        return "{hostname}/devices/{device_id}".format(hostname=hostname, device_id=device_id)
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -91,31 +98,52 @@ class AbstractIoTHubClient(object):
             arbitrary product info which is appended to the user agent string.
         :param proxy_options: Options for sending traffic through proxy servers.
         :type proxy_options: :class:`azure.iot.device.ProxyOptions`
+        :param int sastoken_ttl: The time to live (in seconds) for the created SasToken used for
+            authentication. Default is 3600 seconds (1 hour)
 
         :raises: ValueError if given an invalid connection_string.
-        :raises: TypeError if given an unrecognized parameter.
+        :raises: TypeError if given an unsupported parameter.
 
         :returns: An instance of an IoTHub client that uses a connection string for authentication.
         """
         # TODO: Make this device/module specific and reject non-matching connection strings.
-        # This will require refactoring of the auth package to use common objects (e.g. ConnectionString)
-        # in order to differentiate types of connection strings.
 
+        # Ensure no invalid kwargs were passed by the user
         _validate_kwargs(**kwargs)
 
+        # Create SasToken
+        connection_string = cs.ConnectionString(connection_string)
+        uri = _form_sas_uri(
+            hostname=connection_string[cs.HOST_NAME],
+            device_id=connection_string[cs.DEVICE_ID],
+            module_id=connection_string.get(cs.MODULE_ID),
+        )
+        signing_mechanism = auth.SymmetricKeySigningMechanism(
+            key=connection_string[cs.SHARED_ACCESS_KEY]
+        )
+        token_ttl = kwargs.get("sastoken_ttl", 3600)
+        try:
+            sastoken = st.SasToken(uri, signing_mechanism, ttl=token_ttl)
+        except st.SasTokenError as e:
+            new_err = ValueError("Could not create a SasToken using provided values")
+            new_err.__cause__ = e
+            raise new_err
         # Pipeline Config setup
-        pipeline_config_kwargs = _get_pipeline_config_kwargs(**kwargs)
-        pipeline_configuration = pipeline.IoTHubPipelineConfig(**pipeline_config_kwargs)
+        config_kwargs = _get_config_kwargs(**kwargs)
+        pipeline_configuration = pipeline.IoTHubPipelineConfig(
+            device_id=connection_string[cs.DEVICE_ID],
+            module_id=connection_string.get(cs.MODULE_ID),
+            hostname=connection_string[cs.HOST_NAME],
+            gateway_hostname=connection_string.get(cs.GATEWAY_HOST_NAME),
+            sastoken=sastoken,
+            **config_kwargs
+        )
         if cls.__name__ == "IoTHubDeviceClient":
             pipeline_configuration.blob_upload = True
 
-        # Auth Provider setup
-        authentication_provider = auth.SymmetricKeyAuthenticationProvider.parse(connection_string)
-        authentication_provider.server_verification_cert = kwargs.get("server_verification_cert")
-
         # Pipeline setup
-        http_pipeline = pipeline.HTTPPipeline(authentication_provider, pipeline_configuration)
-        mqtt_pipeline = pipeline.MQTTPipeline(authentication_provider, pipeline_configuration)
+        http_pipeline = pipeline.HTTPPipeline(pipeline_configuration)
+        mqtt_pipeline = pipeline.MQTTPipeline(pipeline_configuration)
 
         return cls(mqtt_pipeline, http_pipeline)
 
@@ -188,26 +216,24 @@ class AbstractIoTHubDeviceClient(AbstractIoTHubClient):
         :param proxy_options: Options for sending traffic through proxy servers.
         :type proxy_options: :class:`azure.iot.device.ProxyOptions`
 
-        :raises: TypeError if given an unrecognized parameter.
+        :raises: TypeError if given an unsupported parameter.
 
         :returns: An instance of an IoTHub client that uses an X509 certificate for authentication.
         """
-        _validate_kwargs(**kwargs)
+        # Ensure no invalid kwargs were passed by the user
+        excluded_kwargs = ["sastoken_ttl"]
+        _validate_kwargs(exclude=excluded_kwargs, **kwargs)
 
         # Pipeline Config setup
-        pipeline_config_kwargs = _get_pipeline_config_kwargs(**kwargs)
-        pipeline_configuration = pipeline.IoTHubPipelineConfig(**pipeline_config_kwargs)
+        config_kwargs = _get_config_kwargs(**kwargs)
+        pipeline_configuration = pipeline.IoTHubPipelineConfig(
+            device_id=device_id, hostname=hostname, x509=x509, **config_kwargs
+        )
         pipeline_configuration.blob_upload = True  # Blob Upload is a feature on Device Clients
 
-        # Auth Provider setup
-        authentication_provider = auth.X509AuthenticationProvider(
-            x509=x509, hostname=hostname, device_id=device_id
-        )
-        authentication_provider.server_verification_cert = kwargs.get("server_verification_cert")
-
         # Pipeline setup
-        http_pipeline = pipeline.HTTPPipeline(authentication_provider, pipeline_configuration)
-        mqtt_pipeline = pipeline.MQTTPipeline(authentication_provider, pipeline_configuration)
+        http_pipeline = pipeline.HTTPPipeline(pipeline_configuration)
+        mqtt_pipeline = pipeline.MQTTPipeline(pipeline_configuration)
 
         return cls(mqtt_pipeline, http_pipeline)
 
@@ -233,27 +259,37 @@ class AbstractIoTHubDeviceClient(AbstractIoTHubClient):
             arbitrary product info which is appended to the user agent string.
         :param proxy_options: Options for sending traffic through proxy servers.
         :type proxy_options: :class:`azure.iot.device.ProxyOptions`
+        :param int sastoken_ttl: The time to live (in seconds) for the created SasToken used for
+            authentication. Default is 3600 seconds (1 hour)
 
-        :raises: TypeError if given an unrecognized parameter.
+        :raises: TypeError if given an unsupported parameter.
 
         :return: An instance of an IoTHub client that uses a symmetric key for authentication.
         """
+        # Ensure no invalid kwargs were passed by the user
         _validate_kwargs(**kwargs)
 
+        # Create SasToken
+        uri = _form_sas_uri(hostname=hostname, device_id=device_id)
+        signing_mechanism = auth.SymmetricKeySigningMechanism(key=symmetric_key)
+        token_ttl = kwargs.get("sastoken_ttl", 3600)
+        try:
+            sastoken = st.SasToken(uri, signing_mechanism, ttl=token_ttl)
+        except st.SasTokenError as e:
+            new_err = ValueError("Could not create a SasToken using provided values")
+            new_err.__cause__ = e
+            raise new_err
+
         # Pipeline Config setup
-        pipeline_config_kwargs = _get_pipeline_config_kwargs(**kwargs)
-        pipeline_configuration = pipeline.IoTHubPipelineConfig(**pipeline_config_kwargs)
+        config_kwargs = _get_config_kwargs(**kwargs)
+        pipeline_configuration = pipeline.IoTHubPipelineConfig(
+            device_id=device_id, hostname=hostname, sastoken=sastoken, **config_kwargs
+        )
         pipeline_configuration.blob_upload = True  # Blob Upload is a feature on Device Clients
 
-        # Auth Provider setup
-        authentication_provider = auth.SymmetricKeyAuthenticationProvider(
-            hostname=hostname, device_id=device_id, module_id=None, shared_access_key=symmetric_key
-        )
-        authentication_provider.server_verification_cert = kwargs.get("server_verification_cert")
-
         # Pipeline setup
-        http_pipeline = pipeline.HTTPPipeline(authentication_provider, pipeline_configuration)
-        mqtt_pipeline = pipeline.MQTTPipeline(authentication_provider, pipeline_configuration)
+        http_pipeline = pipeline.HTTPPipeline(pipeline_configuration)
+        mqtt_pipeline = pipeline.MQTTPipeline(pipeline_configuration)
 
         return cls(mqtt_pipeline, http_pipeline)
 
@@ -289,18 +325,19 @@ class AbstractIoTHubModuleClient(AbstractIoTHubClient):
             arbitrary product info which is appended to the user agent string.
         :param proxy_options: Options for sending traffic through proxy servers.
         :type proxy_options: :class:`azure.iot.device.ProxyOptions`
+        :param int sastoken_ttl: The time to live (in seconds) for the created SasToken used for
+            authentication. Default is 3600 seconds (1 hour)
 
         :raises: OSError if the IoT Edge container is not configured correctly.
         :raises: ValueError if debug variables are invalid.
+        :raises: TypeError if given an unsupported parameter.
 
         :returns: An instance of an IoTHub client that uses the IoT Edge environment for
             authentication.
         """
-        _validate_kwargs(**kwargs)
-        if kwargs.get("server_verification_cert"):
-            raise TypeError(
-                "'server_verification_cert' is not supported by clients using an IoT Edge environment"
-            )
+        # Ensure no invalid kwargs were passed by the user
+        excluded_kwargs = ["server_verification_cert"]
+        _validate_kwargs(exclude=excluded_kwargs, **kwargs)
 
         # First try the regular Edge container variables
         try:
@@ -322,8 +359,9 @@ class AbstractIoTHubModuleClient(AbstractIoTHubClient):
                 new_err = OSError("IoT Edge environment not configured correctly")
                 new_err.__cause__ = e
                 raise new_err
-            # TODO: variant server_verification_cert file vs data object that would remove the need for this fopen
+
             # Read the certificate file to pass it on as a string
+            # TODO: variant server_verification_cert file vs data object that would remove the need for this fopen
             try:
                 with io.open(ca_cert_filepath, mode="r") as ca_cert_file:
                     server_verification_cert = ca_cert_file.read()
@@ -338,41 +376,68 @@ class AbstractIoTHubModuleClient(AbstractIoTHubClient):
                 new_err = ValueError("Invalid CA certificate file")
                 new_err.__cause__ = e
                 raise new_err
-            # Use Symmetric Key authentication for local dev experience.
+
+            # Extract config values from connection string
+            connection_string = cs.ConnectionString(connection_string)
             try:
-                authentication_provider = auth.SymmetricKeyAuthenticationProvider.parse(
-                    connection_string
-                )
-            except ValueError:
-                raise
-            authentication_provider.server_verification_cert = server_verification_cert
+                device_id = connection_string[cs.DEVICE_ID]
+                module_id = connection_string[cs.MODULE_ID]
+                hostname = connection_string[cs.HOST_NAME]
+                gateway_hostname = connection_string[cs.GATEWAY_HOST_NAME]
+            except KeyError:
+                raise ValueError("Invalid Connection String")
+
+            # Use Symmetric Key authentication for local dev experience.
+            signing_mechanism = auth.SymmetricKeySigningMechanism(
+                key=connection_string[cs.SHARED_ACCESS_KEY]
+            )
+
         else:
             # Use an HSM for authentication in the general case
+            hsm = edge_hsm.IoTEdgeHsm(
+                module_id=module_id,
+                generation_id=module_generation_id,
+                workload_uri=workload_uri,
+                api_version=api_version,
+            )
             try:
-                authentication_provider = auth.IoTEdgeAuthenticationProvider(
-                    hostname=hostname,
-                    device_id=device_id,
-                    module_id=module_id,
-                    gateway_hostname=gateway_hostname,
-                    module_generation_id=module_generation_id,
-                    workload_uri=workload_uri,
-                    api_version=api_version,
-                )
-            except auth.IoTEdgeError as e:
+                server_verification_cert = hsm.get_certificate()
+            except edge_hsm.IoTEdgeError as e:
                 new_err = OSError("Unexpected failure in IoTEdge")
                 new_err.__cause__ = e
                 raise new_err
+            signing_mechanism = hsm
+
+        # Create SasToken
+        uri = _form_sas_uri(hostname=hostname, device_id=device_id, module_id=module_id)
+        token_ttl = kwargs.get("sastoken_ttl", 3600)
+        try:
+            sastoken = st.SasToken(uri, signing_mechanism, ttl=token_ttl)
+        except st.SasTokenError as e:
+            new_err = ValueError(
+                "Could not create a SasToken using the values provided, or in the Edge environment"
+            )
+            new_err.__cause__ = e
+            raise new_err
 
         # Pipeline Config setup
-        pipeline_config_kwargs = _get_pipeline_config_kwargs(**kwargs)
-        pipeline_configuration = pipeline.IoTHubPipelineConfig(**pipeline_config_kwargs)
+        config_kwargs = _get_config_kwargs(**kwargs)
+        pipeline_configuration = pipeline.IoTHubPipelineConfig(
+            device_id=device_id,
+            module_id=module_id,
+            hostname=hostname,
+            gateway_hostname=gateway_hostname,
+            sastoken=sastoken,
+            server_verification_cert=server_verification_cert,
+            **config_kwargs
+        )
         pipeline_configuration.method_invoke = (
             True
         )  # Method Invoke is allowed on modules created from edge environment
 
         # Pipeline setup
-        http_pipeline = pipeline.HTTPPipeline(authentication_provider, pipeline_configuration)
-        mqtt_pipeline = pipeline.MQTTPipeline(authentication_provider, pipeline_configuration)
+        http_pipeline = pipeline.HTTPPipeline(pipeline_configuration)
+        mqtt_pipeline = pipeline.MQTTPipeline(pipeline_configuration)
 
         return cls(mqtt_pipeline, http_pipeline)
 
@@ -404,25 +469,23 @@ class AbstractIoTHubModuleClient(AbstractIoTHubClient):
         :param proxy_options: Options for sending traffic through proxy servers.
         :type proxy_options: :class:`azure.iot.device.ProxyOptions`
 
-        :raises: TypeError if given an unrecognized parameter.
+        :raises: TypeError if given an unsupported parameter.
 
         :returns: An instance of an IoTHub client that uses an X509 certificate for authentication.
         """
-        _validate_kwargs(**kwargs)
+        # Ensure no invalid kwargs were passed by the user
+        excluded_kwargs = ["sastoken_ttl"]
+        _validate_kwargs(exclude=excluded_kwargs, **kwargs)
 
         # Pipeline Config setup
-        pipeline_config_kwargs = _get_pipeline_config_kwargs(**kwargs)
-        pipeline_configuration = pipeline.IoTHubPipelineConfig(**pipeline_config_kwargs)
-
-        # Auth Provider setup
-        authentication_provider = auth.X509AuthenticationProvider(
-            x509=x509, hostname=hostname, device_id=device_id, module_id=module_id
+        config_kwargs = _get_config_kwargs(**kwargs)
+        pipeline_configuration = pipeline.IoTHubPipelineConfig(
+            device_id=device_id, module_id=module_id, hostname=hostname, x509=x509, **config_kwargs
         )
-        authentication_provider.server_verification_cert = kwargs.get("server_verification_cert")
 
         # Pipeline setup
-        http_pipeline = pipeline.HTTPPipeline(authentication_provider, pipeline_configuration)
-        mqtt_pipeline = pipeline.MQTTPipeline(authentication_provider, pipeline_configuration)
+        http_pipeline = pipeline.HTTPPipeline(pipeline_configuration)
+        mqtt_pipeline = pipeline.MQTTPipeline(pipeline_configuration)
         return cls(mqtt_pipeline, http_pipeline)
 
     @abc.abstractmethod
